use sqlx::SqlitePool;

use crate::types::{uuid::Uuid, event_type::EventType};

use super::{rooms::Room, users::User};

#[derive(Debug, PartialEq, Eq, Hash)]
pub struct Event {
    uuid: Uuid,
    room_uuid: Uuid,
    r#type: EventType,
    state_key: Option<String>,
    sender_uuid: Uuid,
    origin_server_ts: i64,
    content: String,
}

impl Event {
    fn new(
        room: &Room,
        r#type: EventType,
        state_key: Option<String>,
        sender: &User,
        origin_server_ts: i64,
        content: &str,
    ) -> anyhow::Result<Self> {
        Ok(Self {
            uuid: uuid::Uuid::new_v4().into(),
            room_uuid: room.uuid().to_owned(),
            state_key,
            r#type,
            sender_uuid: sender.uuid().to_owned(),
            origin_server_ts,
            content: content.to_owned(),
        })
    }

    pub async fn create(&self, conn: &SqlitePool) -> anyhow::Result<Self> {
        Ok(sqlx::query_as!(
            Self,
            "insert into events(uuid, room_uuid, type, state_key, sender_uuid, origin_server_ts, content)
            values(?, ?, ?, ?, ?, ?, ?)
            returning uuid as 'uuid: Uuid', room_uuid as 'room_uuid: Uuid', type as 'type: EventType', state_key, sender_uuid as 'sender_uuid: Uuid', origin_server_ts, content",
            self.uuid,
            self.room_uuid,
            self.r#type,
            self.state_key,
            self.sender_uuid,
            self.origin_server_ts,
            self.content
        )
        .fetch_one(conn)
        .await?)
    }

    pub async fn all_for_room(conn: &SqlitePool, room: &Room) -> anyhow::Result<Vec<Event>> {
        let room_uuid = room.uuid();
        Ok(sqlx::query_as!(
            Self,
            "select uuid as 'uuid: Uuid', room_uuid as 'room_uuid: Uuid', type as 'type: EventType', state_key, sender_uuid as 'sender_uuid: Uuid', origin_server_ts, content
            from events
            where room_uuid = ?",
            room_uuid
        )
        .fetch_all(conn)
        .await?)
    }

    pub fn content(&self) -> serde_json::Value {
        serde_json::from_str(&self.content).expect("has to be valid json")
    }
}
